==> org.apache.sling.scripting.sightly-1.4.28-1.4.0-source-release.zip: org.apache.sling.scripting.sightly-1.4.28-1.4.0/src/test/java/org/apache/sling/scripting/sightly/impl/engine/extension/URIManipulationFilterExtensionTest.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.sling.scripting.sightly.impl.engine.extension;

import javax.script.Bindings;
import javax.script.SimpleBindings;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Collections;
import java.util.LinkedHashMap;

import org.apache.sling.api.uri.SlingUriBuilder;
import org.apache.sling.scripting.sightly.render.AbstractRuntimeObjectModel;
import org.apache.sling.scripting.sightly.render.RenderContext;
import org.apache.sling.scripting.sightly.render.RuntimeObjectModel;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

public class URIManipulationFilterExtensionTest {

    private static RenderContext renderContext;
    private URIManipulationFilterExtension underTest;

    @BeforeClass
    public static void prepareTests() {
        // wire only the AbstractRuntimeObjectModel
        renderContext = new RenderContext() {
            @Override
            public RuntimeObjectModel getObjectModel() {
                return new AbstractRuntimeObjectModel() {};
            }

            @Override
            public Bindings getBindings() {
                return new SimpleBindings();
            }

            @Override
            public Object call(String functionName, Object... arguments) {
                return null;
            }
        };
    }

    @Before
    public void setUp() {
        underTest = new URIManipulationFilterExtension();
    }

    @Test
    public void testUnescapePercentInQuery() throws URISyntaxException {
        URI testUri = new URI("http", null, "example.com", -1, "/example/search.html", "q=6%25-10%25", "fragment=%");
        assertEquals(
                "http://example.com/example/search.html?q=6%25-10%25#fragment=%25",
                URIManipulationFilterExtension.unescapePercentInQuery(testUri.toString()));
        testUri = new URI("http", null, "example.com", -1, "/example/search.html", "q=6%25-10%25", null);
        assertEquals(
                "http://example.com/example/search.html?q=6%25-10%25",
                URIManipulationFilterExtension.unescapePercentInQuery(testUri.toString()));
    }

    @Test
    public void testConcatenateWithSlashes() {
        assertEquals("a/b/c", URIManipulationFilterExtension.concatenateWithSlashes("a", "b", "c"));
        assertEquals("a/b/c", URIManipulationFilterExtension.concatenateWithSlashes("a", "/b", "c"));
        assertEquals("a/b/c", URIManipulationFilterExtension.concatenateWithSlashes("a", "/b", "/c"));
        assertEquals("/b/", URIManipulationFilterExtension.concatenateWithSlashes("/", "b", "/"));
        assertEquals("/one/two/", URIManipulationFilterExtension.concatenateWithSlashes("/one/", "/two/"));
    }

    @Test
    public void testPercentEncodedURLs_SLING_6761() {
        assertEquals(
                "/example/search.html?q=6%25-10%25",
                underTest.call(renderContext, "/example/search?q=6%25-10%25", new LinkedHashMap<String, Object>() {
                    {
                        put(URIManipulationFilterExtension.EXTENSION, "html");
                    }
                }));
        assertEquals(
                "/example/search.a.html?q=6%25-10%25&s=%40sling&t=%25sling",
                underTest.call(renderContext, "/example/search?q=6%25-10%25", new LinkedHashMap<String, Object>() {
                    {
                        put(URIManipulationFilterExtension.EXTENSION, "html");
                        put(URIManipulationFilterExtension.ADD_SELECTORS, "a");
                        put(URIManipulationFilterExtension.ADD_QUERY, new LinkedHashMap<String, Object>() {
                            {
                                put("s", "@sling");
                                put("t", "%sling");
                            }
                        });
                    }
                }));
    }

    @Test
    public void testOpaqueUris() {
        // see SLING_7000
        assertEquals(
                "mailto:test@apache.org",
                underTest.call(renderContext, "mailto:test@apache.org", new LinkedHashMap<String, Object>() {
                    {
                        put(URIManipulationFilterExtension.EXTENSION, "html");
                    }
                }));
        // check other opaque links

        // data url according to https://tools.ietf.org/html/rfc2397
        String dataUrl = "data:image/gif;base64,R0lGODdhMAAwAPAAAAAAAP///ywAAAAAMAAw"
                + "AAAC8IyPqcvt3wCcDkiLc7C0qwyGHhSWpjQu5yqmCYsapyuvUUlvONmOZtfzgFz"
                + "ByTB10QgxOR0TqBQejhRNzOfkVJ+5YiUqrXF5Y5lKh/DeuNcP5yLWGsEbtLiOSp"
                + "a/TPg7JpJHxyendzWTBfX0cxOnKPjgBzi4diinWGdkF8kjdfnycQZXZeYGejmJl"
                + "ZeGl9i2icVqaNVailT6F5iJ90m6mvuTS4OK05M0vDk0Q4XUtwvKOzrcd3iq9uis"
                + "F81M1OIcR7lEewwcLp7tuNNkM3uNna3F2JQFo97Vriy/Xl4/f1cf5VWzXyym7PH"
                + "hhx4dbgYKAAA7";

        assertEquals(dataUrl, underTest.call(renderContext, dataUrl, new LinkedHashMap<String, Object>() {
            {
                put(URIManipulationFilterExtension.EXTENSION, "html");
            }
        }));
    }

    @Test
    public void testHandleSelectorsEmptyOptions() {
        final SlingUriBuilder builder = SlingUriBuilder.create().setPath("/content.s1.s2.html");

        underTest.handleSelectors(null, builder, Collections.emptyMap());
        assertEquals("s1.s2", builder.getSelectorString());
    }

    @Test
    public void testHandleSelectorsNullSelectors() {
        final SlingUriBuilder builder = SlingUriBuilder.create().setPath("/content.s1.s2.html");

        underTest.handleSelectors(null, builder, Collections.singletonMap("selectors", null));
        assertNull(builder.getSelectorString());
    }

    @Test
    public void testHandleSelectorsStringSelectors() {
        final SlingUriBuilder builder = SlingUriBuilder.create().setPath("/content.s1.s2.html");

        underTest.handleSelectors(null, builder, Collections.singletonMap("selectors", "a.b"));
        assertEquals("a.b", builder.getSelectorString());
    }

    @Test
    public void testHandleSelectorsStringArraySelectors() {
        final SlingUriBuilder builder = SlingUriBuilder.create().setPath("/content.s1.s2.html");

        final RuntimeObjectModel rom = new AbstractRuntimeObjectModel() {};
        underTest.handleSelectors(rom, builder, Collections.singletonMap("selectors", new String[] {"a.b"}));
        assertEquals("a.b", builder.getSelectorString());
    }

    @Test
    public void testHandleSelectorsStringAddSelectors() {
        final SlingUriBuilder builder = SlingUriBuilder.create().setPath("/content.s1.s2.html");

        underTest.handleSelectors(null, builder, Collections.singletonMap("addSelectors", "a.b"));
        assertEquals("s1.s2.a.b", builder.getSelectorString());
    }

    @Test
    public void testHandleSelectorsStringArrayAddSelectors() {
        final SlingUriBuilder builder = SlingUriBuilder.create().setPath("/content.s1.s2.html");

        final RuntimeObjectModel rom = new AbstractRuntimeObjectModel() {};
        underTest.handleSelectors(rom, builder, Collections.singletonMap("addSelectors", new String[] {"a", "b"}));
        assertEquals("s1.s2.a.b", builder.getSelectorString());
    }

    @Test
    public void testHandleSelectorsStringRemoveSelectors() {
        final SlingUriBuilder builder = SlingUriBuilder.create().setPath("/content.s1.s2.html");

        underTest.handleSelectors(null, builder, Collections.singletonMap("removeSelectors", "a.b.s1"));
        assertEquals("s2", builder.getSelectorString());
    }

    @Test
    public void testHandleSelectorsStringArrayRemoveSelectors() {
        final SlingUriBuilder builder = SlingUriBuilder.create().setPath("/content.s1.s2.html");

        final RuntimeObjectModel rom = new AbstractRuntimeObjectModel() {};
        underTest.handleSelectors(
                rom, builder, Collections.singletonMap("removeSelectors", new String[] {"a", "b", "s1"}));
        assertEquals("s2", builder.getSelectorString());
    }
}

==> org.apache.sling.scripting.sightly-2.0.0-1.4.0-source-release.zip: org.apache.sling.scripting.sightly-2.0.0-1.4.0/src/test/java/org/apache/sling/scripting/sightly/impl/engine/extension/URIManipulationFilterExtensionTest.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.sling.scripting.sightly.impl.engine.extension;

import javax.script.Bindings;
import javax.script.SimpleBindings;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Collections;
import java.util.LinkedHashMap;

import org.apache.sling.api.uri.SlingUriBuilder;
import org.apache.sling.scripting.sightly.render.AbstractRuntimeObjectModel;
import org.apache.sling.scripting.sightly.render.RenderContext;
import org.apache.sling.scripting.sightly.render.RuntimeObjectModel;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

public class URIManipulationFilterExtensionTest {

    private static RenderContext renderContext;
    private URIManipulationFilterExtension underTest;

    @BeforeClass
    public static void prepareTests() {
        // wire only the AbstractRuntimeObjectModel
        renderContext = new RenderContext() {
            @Override
            public RuntimeObjectModel getObjectModel() {
                return new AbstractRuntimeObjectModel() {};
            }

            @Override
            public Bindings getBindings() {
                return new SimpleBindings();
            }

            @Override
            public Object call(String functionName, Object... arguments) {
                return null;
            }
        };
    }

    @Before
    public void setUp() {
        underTest = new URIManipulationFilterExtension();
    }

    @Test
    public void testUnescapePercentInQuery() throws URISyntaxException {
        URI testUri = new URI("http", null, "example.com", -1, "/example/search.html", "q=6%25-10%25", "fragment=%");
        assertEquals(
                "http://example.com/example/search.html?q=6%25-10%25#fragment=%25",
                URIManipulationFilterExtension.unescapePercentInQuery(testUri.toString()));
        testUri = new URI("http", null, "example.com", -1, "/example/search.html", "q=6%25-10%25", null);
        assertEquals(
                "http://example.com/example/search.html?q=6%25-10%25",
                URIManipulationFilterExtension.unescapePercentInQuery(testUri.toString()));
    }

    @Test
    public void testConcatenateWithSlashes() {
        assertEquals("a/b/c", URIManipulationFilterExtension.concatenateWithSlashes("a", "b", "c"));
        assertEquals("a/b/c", URIManipulationFilterExtension.concatenateWithSlashes("a", "/b", "c"));
        assertEquals("a/b/c", URIManipulationFilterExtension.concatenateWithSlashes("a", "/b", "/c"));
        assertEquals("/b/", URIManipulationFilterExtension.concatenateWithSlashes("/", "b", "/"));
        assertEquals("/one/two/", URIManipulationFilterExtension.concatenateWithSlashes("/one/", "/two/"));
    }

    @Test
    public void testPercentEncodedURLs_SLING_6761() {
        assertEquals(
                "/example/search.html?q=6%25-10%25",
                underTest.call(renderContext, "/example/search?q=6%25-10%25", new LinkedHashMap<String, Object>() {
                    private static final long serialVersionUID = 1L;

                    {
                        put(URIManipulationFilterExtension.EXTENSION, "html");
                    }
                }));
        assertEquals(
                "/example/search.a.html?q=6%25-10%25&s=%40sling&t=%25sling",
                underTest.call(renderContext, "/example/search?q=6%25-10%25", new LinkedHashMap<String, Object>() {
                    private static final long serialVersionUID = 1L;

                    {
                        put(URIManipulationFilterExtension.EXTENSION, "html");
                        put(URIManipulationFilterExtension.ADD_SELECTORS, "a");
                        put(URIManipulationFilterExtension.ADD_QUERY, new LinkedHashMap<String, Object>() {
                            private static final long serialVersionUID = 1L;

                            {
                                put("s", "@sling");
                                put("t", "%sling");
                            }
                        });
                    }
                }));
    }

    @Test
    public void testOpaqueUris() {
        // see SLING_7000
        assertEquals(
                "mailto:test@apache.org",
                underTest.call(renderContext, "mailto:test@apache.org", new LinkedHashMap<String, Object>() {
                    private static final long serialVersionUID = 1L;

                    {
                        put(URIManipulationFilterExtension.EXTENSION, "html");
                    }
                }));
        // check other opaque links

        // data url according to https://tools.ietf.org/html/rfc2397
        String dataUrl = "data:image/gif;base64,R0lGODdhMAAwAPAAAAAAAP///ywAAAAAMAAw"
                + "AAAC8IyPqcvt3wCcDkiLc7C0qwyGHhSWpjQu5yqmCYsapyuvUUlvONmOZtfzgFz"
                + "ByTB10QgxOR0TqBQejhRNzOfkVJ+5YiUqrXF5Y5lKh/DeuNcP5yLWGsEbtLiOSp"
                + "a/TPg7JpJHxyendzWTBfX0cxOnKPjgBzi4diinWGdkF8kjdfnycQZXZeYGejmJl"
                + "ZeGl9i2icVqaNVailT6F5iJ90m6mvuTS4OK05M0vDk0Q4XUtwvKOzrcd3iq9uis"
                + "F81M1OIcR7lEewwcLp7tuNNkM3uNna3F2JQFo97Vriy/Xl4/f1cf5VWzXyym7PH"
                + "hhx4dbgYKAAA7";

        assertEquals(dataUrl, underTest.call(renderContext, dataUrl, new LinkedHashMap<String, Object>() {
            private static final long serialVersionUID = 1L;

            {
                put(URIManipulationFilterExtension.EXTENSION, "html");
            }
        }));
    }

    @Test
    public void testHandleSelectorsEmptyOptions() {
        final SlingUriBuilder builder = SlingUriBuilder.create().setPath("/content.s1.s2.html");

        underTest.handleSelectors(null, builder, Collections.emptyMap());
        assertEquals("s1.s2", builder.getSelectorString());
    }

    @Test
    public void testHandleSelectorsNullSelectors() {
        final SlingUriBuilder builder = SlingUriBuilder.create().setPath("/content.s1.s2.html");

        underTest.handleSelectors(null, builder, Collections.singletonMap("selectors", null));
        assertNull(builder.getSelectorString());
    }

    @Test
    public void testHandleSelectorsStringSelectors() {
        final SlingUriBuilder builder = SlingUriBuilder.create().setPath("/content.s1.s2.html");

        underTest.handleSelectors(null, builder, Collections.singletonMap("selectors", "a.b"));
        assertEquals("a.b", builder.getSelectorString());
    }

    @Test
    public void testHandleSelectorsStringArraySelectors() {
        final SlingUriBuilder builder = SlingUriBuilder.create().setPath("/content.s1.s2.html");

        final RuntimeObjectModel rom = new AbstractRuntimeObjectModel() {};
        underTest.handleSelectors(rom, builder, Collections.singletonMap("selectors", new String[] {"a.b"}));
        assertEquals("a.b", builder.getSelectorString());
    }

    @Test
    public void testHandleSelectorsStringAddSelectors() {
        final SlingUriBuilder builder = SlingUriBuilder.create().setPath("/content.s1.s2.html");

        underTest.handleSelectors(null, builder, Collections.singletonMap("addSelectors", "a.b"));
        assertEquals("s1.s2.a.b", builder.getSelectorString());
    }

    @Test
    public void testHandleSelectorsStringArrayAddSelectors() {
        final SlingUriBuilder builder = SlingUriBuilder.create().setPath("/content.s1.s2.html");

        final RuntimeObjectModel rom = new AbstractRuntimeObjectModel() {};
        underTest.handleSelectors(rom, builder, Collections.singletonMap("addSelectors", new String[] {"a", "b"}));
        assertEquals("s1.s2.a.b", builder.getSelectorString());
    }

    @Test
    public void testHandleSelectorsStringRemoveSelectors() {
        final SlingUriBuilder builder = SlingUriBuilder.create().setPath("/content.s1.s2.html");

        underTest.handleSelectors(null, builder, Collections.singletonMap("removeSelectors", "a.b.s1"));
        assertEquals("s2", builder.getSelectorString());
    }

    @Test
    public void testHandleSelectorsStringArrayRemoveSelectors() {
        final SlingUriBuilder builder = SlingUriBuilder.create().setPath("/content.s1.s2.html");

        final RuntimeObjectModel rom = new AbstractRuntimeObjectModel() {};
        underTest.handleSelectors(
                rom, builder, Collections.singletonMap("removeSelectors", new String[] {"a", "b", "s1"}));
        assertEquals("s2", builder.getSelectorString());
    }
}
